#!/bin/bash

#
# builds packages from given dir
#

# $Header: /magellan-cvs/magellan-src/mage/usr/lib/mage/pkgbuild_dir.sh,v 1.15 2007-03-19 19:24:46 niro Exp $

# set default user mage.rc
: ${MAGERC="/etc/mage.rc"}

PKGSUFFIX="mpk"
PKGNAME="$1"
SEARCHDIRS="$2"
MLIBDIR=/usr/lib/mage

## only for tests -> normally in /etc/rc.d/init.d/functions
COLRED="\033[1;6m\033[31m"
COLGREEN="\033[1;6m\033[32m"
COLYELLOW="\033[1;6m\033[33m"
COLBLUE="\033[1;6m\033[34m"
COLMAGENTA="\033[1;6m\033[35m"
COLWHITE="\033[1;6m\033[37m"
COLGRAY="\033[0;6m\033[37m"
COLBOLD="\033[1m"
COLDEFAULT="\033[0m"

if [[ ${NOCOLORS} = true ]]
then
	COLRED=""
	COLGREEN=""
	COLYELLOW=""
	COLBLUE=""
	COLMAGENTA=""
	COLWHITE=""
	COLGRAY=""
	COLBOLD=""
	COLDEFAULT=""
fi

# export default C locale
export LC_ALL=C

[[ -z $1 ]] && echo "No package name given. Aborting." && exit 1
[[ -z $2 ]] && echo "No source dir given. Aborting." && exit 1

source /etc/mage.rc.global
[[ -f ${MAGERC} ]] && source ${MAGERC}
source ${MLIBDIR}/mage4.functions.sh

# set PKGDIR and BUILDDIR and BINDIR to MROOT
if [[ -n ${MROOT} ]]
then
	export PKGDIR=${MROOT}/${PKGDIR}
	export BUILDDIR=${MROOT}/${BUILDDIR}
	export BINDIR=${MROOT}/${BINDIR}
fi

build_preinstall()
{
	if [ -d ${BUILDDIR}/${PKGNAME} ]
	then
		rm -rf ${BUILDDIR}/${PKGNAME}
	fi

	install -d ${BUILDDIR}/${PKGNAME}/binfiles
	touch ${BUILDDIR}/${PKGNAME}/.dirs
	touch ${BUILDDIR}/${PKGNAME}/.symlinks
	touch ${BUILDDIR}/${PKGNAME}/.files
	touch ${BUILDDIR}/${PKGNAME}/.pipes
	touch ${BUILDDIR}/${PKGNAME}/.char
}

build_postinstall()
{
	echo -e "${COLBLUE}===${COLGREEN} fetching files for package '${PKGNAME}' ...${COLDEFAULT}"

	# md5sums will only be generated for files: $M_MD5SUM

	# sets mtime to same value of the $BINDIR: $M_MTIME
	# only needed for files and symlinks
	M_MTIME=$(stat -c %Y ${BINDIR})

	# install mtime to package (needed for later checks)
	echo "${M_MTIME}" > ${BUILDDIR}/${PKGNAME}/.mtime

	# !! we use  as field seperator !!
	# doing so prevent us to get errors by filenames with spaces
	BUILD_TODELFILES=$(find ${SEARCHDIRS} -printf %p)

	# sets fieldseperator to "" instead of " "
	IFS=

	for I in $BUILD_TODELFILES
	do
		CHECK_SRCDIR="$(echo ${I#${SEARCHDIRS}*}|grep $BUILDDIR)"

		if [ -z "$CHECK_SRCDIR" ]
		then
			FILETYPE="$(stat -c %F $I)"
			FILEPOSIX="$(stat -c %a $I)"
			FILEOWNER="$(stat -c %U $I)"
			FILEGROUP="$(stat -c %G $I)"

			[[ ${FILEOWNER} = UNKNOWN ]] && FILEOWNER="root"
			[[ ${FILEGROUP} = UNKNOWN ]] && FILEGROUP="root"

			case "${FILETYPE}" in
				"directory")
					if [[ $I != ${SEARCHDIRS} ]]
					then
						echo "${I#${SEARCHDIRS}*}${FILEPOSIX}${FILEOWNER}${FILEGROUP}" >> ${BUILDDIR}/${PKGNAME}/.dirs
					fi
					;;

				"symbolic link")
					LNKDEST="$(readlink $I)"
					echo "${I#${SEARCHDIRS}*}${FILEPOSIX}${LNKDEST#${SEARCHDIRS}*}${M_MTIME}" >> ${BUILDDIR}/${PKGNAME}/.symlinks
					;;
					

				"regular empty file")
					FILEDIR="$(dirname $I)"
					FILEDIR="${FILEDIR#${SEARCHDIRS}*}"
					M_MD5SUM="$(md5sum ${I}|cut -d' ' -f1)"
					echo "${I#${SEARCHDIRS}*}${FILEPOSIX}${FILEOWNER}${FILEGROUP}${M_MTIME}${M_MD5SUM}" >> ${BUILDDIR}/${PKGNAME}/.files
					install -d ${BUILDDIR}/${PKGNAME}/binfiles/${FILEDIR}
					cp ${I} ${BUILDDIR}/${PKGNAME}/binfiles/${FILEDIR}
					;;

				"regular file" )
					FILEDIR="$(dirname $I)"
					FILEDIR="${FILEDIR#${SEARCHDIRS}*}"
					M_MD5SUM="$(md5sum ${I}|cut -d' ' -f1)"
					echo "${I#${SEARCHDIRS}*}${FILEPOSIX}${FILEOWNER}${FILEGROUP}${M_MTIME}${M_MD5SUM}" >> ${BUILDDIR}/${PKGNAME}/.files
					install -d ${BUILDDIR}/${PKGNAME}/binfiles/${FILEDIR}
					cp ${I} ${BUILDDIR}/${PKGNAME}/binfiles/${FILEDIR}
					;;

				"block special file")
					echo "${I#${SEARCHDIRS}*}${FILEPOSIX}" >> ${BUILDDIR}/${PKGNAME}/.pipes
					;;

				"character special file")
					# convert hex2dec
					#  printf '%d' 0x12 -> 18
					local CHAR_MAJOR="$(printf 0x$(stat -c %t $I))"
					local CHAR_MINOR="$(printf 0x$(stat -c %T $I))"
					echo "${I#${SEARCHDIRS}*}${FILEPOSIX}${CHAR_MAJOR}${CHAR_MINOR}" >> ${BUILDDIR}/${PKGNAME}/.char
					;;

				*)
					echo "What I am ? -- $I"
					echo "$0 paused ... Press Enter"
					read
					;;
			esac
		fi
	done

	# very important: unsetting the '' fieldseperator
	unset IFS

	# forcing mtime to same value
	echo -e "${COLBLUE}===${COLGREEN} forcing mtime to the same value ...${COLDEFAULT}"
	find ${BUILDDIR}/${PKGNAME}/binfiles -exec touch -m -r ${BUILDDIR}/${PKGNAME}/.mtime '{}' ';'
}

build_rmtimestamp()
{
	rm /var/tmp/timestamp
}

build_package()
{
	echo -e "${COLBLUE}===${COLGREEN} building package tarball ...${COLDEFAULT}"
	cd ${BUILDDIR}
	tar cvjf ${PKGNAME}.tar.bz2 ./${PKGNAME}
	install -d ${PKGDIR}
	mv ${PKGNAME}.tar.bz2 ${PKGDIR}/${PKGNAME}.${PKGSUFFIX}
}

build_preinstall
build_postinstall
build_package
